/*! @file
  @author Shin'ichiro Nakaoka
*/

#include "CosNaming_impl.h"
#include "MessageView_impl.h"
#include "NameServerView.h"
#include <cnoid/Plugin>
#include <cnoid/MessageView>
#include <cnoid/CorbaUtil>
#include <cnoid/Sleep>
#include <QTcpSocket>
#include <boost/thread/thread.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/thread/condition_variable.hpp>
#include <boost/bind.hpp>
#include "gettext.h"

using namespace std;
using namespace cnoid;
using namespace boost;

namespace cnoid {
    PortableServer::POA_var nspoa;
}

namespace {
    
    class CorbaPlugin : public Plugin
    {
        boost::thread orbMainLoopThread;
        
    public:
        CorbaPlugin() : Plugin("Corba") { }
        
        virtual bool initialize() {

            // check if the CORBA iiop port needed for getting the existing NameServer can be accessed
            QTcpSocket socket;
            socket.connectToHost("localhost", 2809);

            if(socket.waitForConnected(50)){
                // use the existing name server
                initializeCorbaUtil(true);

            } else {
                // setup the built-in name server
                initializeCorbaUtil(true, 2809);

                CORBA::Object_var obj  = getORB()->resolve_initial_references("omniINSPOA");
                nspoa = PortableServer::POA::_narrow(obj);
                PortableServer::POAManager_var poaManager = nspoa->the_POAManager();
                poaManager->activate();
                PortableServer::ObjectId_var refid = PortableServer::string_to_ObjectId("NameService");
                NamingContext_impl* nc = new NamingContext_impl(nspoa, refid);
                nc->_remove_ref();
                
                MessageView::instance()->putln(_("Built-in CORBA name server has been invoked."));
            }

            MessageView_impl* messageView = new MessageView_impl(getORB());
            getDefaultNamingContextHelper()->bindObject(messageView->_this(), "MessageView");

            addView(new NameServerView());

            orbMainLoopThread = thread(bind(&CorbaPlugin::orbMainLoop, this));

            return true;
        }

        void orbMainLoop() {
            getORB()->run();
            getORB()->destroy();
        }
        
        virtual bool finalize() {
            getORB()->shutdown(false);
            orbMainLoopThread.join();
            return true;
        }
    };
}

CNOID_IMPLEMENT_PLUGIN_ENTRY(CorbaPlugin);
